/**
 * raw_api_sample
 * This JavaScript file is for Home view.
 */

// Viewerの愛読書リスト
var bookList = [];

// ガジェットの初期化
function init() {
	var req = opensocial.newDataRequest();

	// Viewerを取得
	var viewerId = opensocial.IdSpec.PersonId.VIEWER;
	req.add(req.newFetchPersonRequest(viewerId), 'viewer');

	// フレンドを取得
	var friendsIdParams = {};
	friendsIdParams[opensocial.IdSpec.Field.USER_ID]  = viewerId;
	friendsIdParams[opensocial.IdSpec.Field.GROUP_ID] = 'FRIENDS';
	var friendsIdSpec = opensocial.newIdSpec(friendsIdParams);
	req.add(req.newFetchPeopleRequest(friendsIdSpec), 'friends');

	// Viewerの愛読書リストを取得
	var reqParams = {};
	reqParams[opensocial.DataRequest.DataRequestFields.ESCAPE_TYPE] =
		opensocial.EscapeType.NONE;

	var viewerIdParams = {};
	viewerIdParams[opensocial.IdSpec.Field.USER_ID]  = viewerId;
	viewerIdParams[opensocial.IdSpec.Field.GROUP_ID] = 'SELF';
	var viewerIdSpec = opensocial.newIdSpec(viewerIdParams);
	req.add(req.newFetchPersonAppDataRequest(
				viewerIdSpec, ['bookList'], reqParams), 'viewerBooks');

	// フレンドの愛読書リストを取得
	req.add(req.newFetchPersonAppDataRequest(
			friendsIdSpec, ['bookList'], reqParams), 'friendsBooks');

	// リクエストを送信
	req.send(receiveData);
}

// Viewer/フレンド情報のレスポンス処理
function receiveData(data) {
	if(isError(data))
		return;

	// Viewerの愛読書リストを表示
	if(!isError(data.get('viewer')) && !isError(data.get('viewerBooks'))) {
		var viewerId = data.get('viewer').getData().getId();
		bookList = gadgets.json.parse(
			(data.get('viewerBooks').getData()[viewerId] || {})['bookList'] || "[]");
		document.getElementById('mybooks').innerHTML = renderBookList(bookList);
	}

	// フレンドの愛読書リストを表示
	if(!isError(data.get('friends')) && !isError(data.get('friendsBooks'))) {
		var appData = data.get('friendsBooks').getData();
		var html    = [];
		data.get('friends').getData().each(function(person) {
			var books = gadgets.json.parse(
				(appData[person.getId()] || {})['bookList'] || "[]");
			if(books.length > 0) {
				html.push(
					'<div class="person">' +
					'<div class="name">' + person.getDisplayName() + '</div>' +
					'<div class="books">' + renderBookList(books) + '</div>' +
					'</div>');
			}
		});
		document.getElementById('friends').innerHTML = html.join('');
	}

	// ガジェットの高さを調整
	gadgets.window.adjustHeight();
}

// 「愛読書を追加」ボタンの処理
function onSubmit() {

	var bookName = document.getElementById('book-field').value;
	if(!/[^\s]/.test(bookName))
		return;

	// 書籍名からGoogle Book Search APIのリクエストURLを生成
	var url =
		'http://books.google.com/books/feeds/volumes?' +
		'max-results=1&alt=json&q=intitle:' +
		encodeURIComponent('"' + bookName + '"');

	// Google Book Search APIを呼び出す
	var params = {};
	params[gadgets.io.RequestParameters.CONTENT_TYPE] = gadgets.io.ContentType.JSON;
	gadgets.io.makeRequest(url, receiveBookData, params);
}

// Google Book Search APIのレスポンスを処理
function receiveBookData(response) {
	if(response.errors.length <= 0) {
	    var book = extractBookInfo(response.data);
	    if(book) {
			bookList.push(book);
			saveBookList();
			return;
	    }
	}
	alert('書籍の取得に失敗しました。');
}

// 愛読書リストをAppDataに保存
function saveBookList() {
	var json = gadgets.json.stringify(bookList);
	var req  = opensocial.newDataRequest();
	req.add(req.newUpdatePersonAppDataRequest(
				opensocial.IdSpec.PersonId.VIEWER, "bookList", json), 'bookList');
	req.send(function(data) {
		if(!isError(data) && !isError(data.get('bookList'))) {
			document.getElementById('mybooks').innerHTML =
				renderBookList(bookList);
			gadgets.window.adjustHeight();
			postActivity('愛読書の追加', bookList[bookList.length-1].name);
		} else {
			bookList.pop();
			alert('愛読書の追加に失敗しました。');
		}
	})
}

// 愛読書リストのHTML生成
function renderBookList(bookList) {
	var html = [];
	for(var i = 0 ; i < bookList.length ; ++i) {
		var book = bookList[i];
		html.push(
			'<li><a target="_blank" href="' +
			gadgets.util.escapeString(book.url || '') + '">' +
			gadgets.util.escapeString(book.name || '') + '</a></li>');
	}
	return '<ul>' + html.join('') + '</ul>';
}

// アクティビティの送信
function postActivity(title, body) {
	var params = {};
	params[opensocial.Activity.Field.TITLE] = title;
	params[opensocial.Activity.Field.BODY]  = body;
	var activity = opensocial.newActivity(params);
	opensocial.requestCreateActivity(
		activity, opensocial.CreateActivityPriority.LOW);
}
